import dash
from dash import dcc, html
import numpy as np
from dash.dependencies import Output, Input, State
from datetime import date
import krakenex
from pykrakenapi import KrakenAPI
from practica_kraken.kraken import Kraken

api = krakenex.API()
k = KrakenAPI(api)

df_assets_info = Kraken.get_asset_info(k)
time_frame = ['1 min', '5 min', '30 min', '1 hora', '4 horas', '24 horas']

external_stylesheets = [
    {
        "href": "https://fonts.googleapis.com/css2?"
                "family=Lato:wght@400;700&display=swap",
        "rel": "stylesheet",
    },
]
app = dash.Dash(__name__, external_stylesheets=external_stylesheets)
app.title = "Análisis de Cryptomonedas"

app.layout = html.Div(
    children=[
        html.Div(
            children=[
                html.P(children="🪙", className="header-emoji"),
                html.H1(
                    children="Análisis de Cryptomonedas",
                    className="header-title"
                ),
                html.P(
                    children="Analizar el comportamiento de un par de cryptomonedas desde Kraken"
                             " mediante el cálculo de su vwap para un tiempo determinado."
                             " Este cálculo del vwap se hará intradía.",
                    className="header-description",
                ),
            ],
            className="header",
        ),
        html.Div(
            children=[
                html.Div(
                    children=[
                        html.Div(children="Moneda A", className="menu-title"),
                        dcc.Dropdown(
                            id="filtro-asset1",
                            options=[
                                {"label": asset1, "value": asset1}
                                for asset1 in np.sort(df_assets_info.altname.unique())
                            ],
                            value='XBT',
                            clearable=False,
                            className="dropdown",
                        ),
                    ]
                ),
                html.Div(
                    children=[
                        html.Div(children="Moneda B", className="menu-title"),
                        dcc.Dropdown(
                            id="filtro-asset2",
                            options=[
                                {"label": asset2, "value": asset2}
                                for asset2 in np.sort(df_assets_info.altname.unique())
                            ],
                            value='USDT',
                            clearable=False,
                            className="dropdown",
                        ),
                    ]
                ),
                html.Div(
                    children=[
                        html.Div(
                            children="Fecha", className="menu-title"),
                        dcc.DatePickerSingle(
                            id="seleccion-fecha",
                            min_date_allowed=date(2021, 1, 1),
                            max_date_allowed=date.today(),
                            date=date(2021, 12, 3)
                        ),
                    ]
                ),
                html.Div(
                    children=[
                        html.Div(children="Intervalo", className="menu-title"),
                        dcc.Dropdown(
                            id="seleccion-intervalo",
                            options=[
                                {"label": frame, "value": frame}
                                for frame in time_frame
                            ],
                            value='5 min',
                            clearable=False,
                            className="dropdown",
                        ),
                    ]
                ),
                html.Div(
                    children=[
                        html.Div(
                            children="Hora Inicio - Fin",
                            className="menu-title"
                        ),
                        dcc.RangeSlider(
                            id='seleccion-hora',
                            min=0,
                            max=24,
                            step=0.5,
                            marks={
                                    0: '0 horas',
                                    8: '8 horas',
                                    16: '16 horas',
                                    24: '24 horas'
                                },
                            value=[8, 9],
                            tooltip={
                                "placement": "bottom",
                                "always_visible": True},
                            className="range-slider"
                        ),
                    ]
                ),
            ],
            className="menu",
        ),
        html.Div(
            children=[
                html.Div(
                    children=dcc.Graph(
                        id="grafica-comun", config={"displayModeBar": False},
                    ),
                    className="card",
                ),
            ],
            className="wrapper",
        ),
    ]
)


# callback y funcion para segun la primera moneda que se escoja, se modifiquen las disponibles en la segunda y solo
# esten aquellas para las que hay mercado
@app.callback(
    Output("filtro-asset2", "options"),
    [Input("filtro-asset1", "value")]
)
def update_filtro_asset2(asset1):
    pairs = k.get_tradable_asset_pairs()
    asset1_pairs = pairs[pairs['altname'].str.startswith(asset1)]
    asset2_list = []
    for pair in np.sort(asset1_pairs.wsname.unique()):
        asset2_list.append(pair.split("/")[1])
    options = [{'label': asset2, 'value': asset2} for asset2 in asset2_list]
    return options


# callback y funcion para que la graficacion sea dinamica en funcion de los parametros escogidos en la visualizacion
@app.callback(
    Output("grafica-comun", "figure"),
    [
        State("filtro-asset1", "value"),
        Input("filtro-asset2", "value"),
        Input("seleccion-fecha", "date"),
        Input("seleccion-intervalo", "value"),
        Input("seleccion-hora", "value"),
    ],
)
def update_charts(asset1, asset2, date, frame, value):
    pair = Kraken.set_pair(k, asset1, asset2)
    unix_from_date, unix_to_date = Kraken.set_dates(k, date, value)
    df_trades = Kraken.get_recent_dates(k, pair, unix_from_date, unix_to_date)
    df_vwap = Kraken.get_df_vwap(df_trades, frame)
    common_chart_figure = {
        "data": [
            {
                "x": df_trades["time"].apply(lambda x: k.unixtime_to_datetime(x)),
                "y": df_trades["price"],
                "type": "lines",
                "name": "price",
                "hovertemplate": "$%{y:.2f}<extra></extra>",
            }, {
                "x": df_vwap['dtime'],
                "y": df_vwap["vwap"],
                "type": "lines",
                "name": "vwap",
                "marker": {'color': '#FF851B'}
            },
        ],
        "layout": {
            "title": {
                "text": "Cotizaciones para el par " + asset1 + '/' + asset2,
                "x": 0.05,
                "xanchor": "left",
            },
            "xaxis": {"fixedrange": True},
            "yaxis": {"fixedrange": True},
            "colorway": ["#17B897"],
        },
    }
    return common_chart_figure


if __name__ == "__main__":
    app.run_server(debug=True)



from datetime import datetime
import time
import pandas as pd
import numpy as np
import math


class Kraken:

    # metodo que obtiene la informacion de todos los assets disponibles en Kraken

    @classmethod
    def get_asset_info(cls, k):
        assets = k.get_asset_info()
        return assets

    # metodo que genera el par tras recibir las dos monedas

    @classmethod
    def set_pair(cls, k, coin1, coin2):
        pair = coin1 + coin2
        return pair

    # metodo que recibe los parametros de Dia e Hora Inicio-Fin y los convierte a unixtime
    @classmethod
    def set_dates(cls, k, date, hours):
        date = date.split("-")
        year = int(date[0])
        month = int(date[1])
        day = int(date[2])
        ini_min = None
        fin_min = None
        if type(hours[0]) == 'float':
            _, ini_hour = math.modf(hours[0])
            ini_min = 30
            if type(hours[1]) == 'float':
                _, fin_hour = math.modf(hours[1])
                fin_min = 30
            else:
                fin_hour = hours[1]
        elif type(hours[1]) == 'float':
            ini_hour = hours[0]
            _, fin_hour = math.modf(hours[1])
            fin_min = 30
        else:
            ini_hour = hours[0]
            fin_hour = hours[1]
        if fin_hour == 24:
            fin_hour = 23
            fin_min = 59
        ini_hour = int(ini_hour)
        fin_hour = int(fin_hour)
        if ini_min is not None:
            from_time = datetime(year, month, day, ini_hour, ini_min)
        else:
            from_time = datetime(year, month, day, ini_hour)
        if fin_min is not None:
            to_time = datetime(year, month, day, fin_hour, fin_min)
        else:
            to_time = datetime(year, month, day, fin_hour)
        unix_from_time = k.datetime_to_unixtime(from_time)
        unix_to_time = k.datetime_to_unixtime(to_time)
        return unix_from_time, unix_to_time

    # metodo que recibe el par, y los unixtime y obtiene con esa informacion los trades desde el unixtime
    # de inicio hasta el unixtime del final

    @classmethod
    def get_recent_dates(cls, k, pair, from_date, to_date):
        df_trades = pd.DataFrame()
        i = 0
        while from_date < to_date:
            if i > 0:
                if i == 10:
                    i = 0
                    time.sleep(5)
                time.sleep(1)
            trades, last = k.get_recent_trades(pair, from_date)
            df_trades = df_trades.append(trades.reset_index(), ignore_index=True)
            from_date = int(last/1e9)
            i += 1
        df_trades = df_trades[df_trades['dtime'] <= k.unixtime_to_datetime(to_date)]
        df_trades.sort_values(by=['dtime'], ascending=True, inplace=True)
        return df_trades

    # metodo que hace el calculo del vwap para los precios y volumenes dados

    @classmethod
    def calculate_vwap(cls, prices, volumes):
        prices = np.array(prices)
        volumes = np.array(volumes)
        calculated_vwap = 0.0
        if len(prices) > 0:
            calculated_vwap = sum(prices * volumes) / sum(volumes)
            return calculated_vwap

    # metodo que recibe los trades para el par indicado y la frecuencia por la que quiere calcular el vwap

    @classmethod
    def get_df_vwap(cls, df_trades, frecuency):
        if frecuency == '1 hora':
            frecuency = '1H'
        if frecuency == '4 horas':
            frecuency = '4H'
        if frecuency == '24 horas':
            frecuency = '24H'
        df_vwap = df_trades.groupby(pd.Grouper(key='dtime', freq=frecuency)).\
            apply(lambda x: cls.calculate_vwap(x['price'], x['volume'])).reset_index(name='vwap')
        return df_vwap



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['practica_kraken']

package_data = \
{'': ['*'], 'practica_kraken': ['assets/*']}

install_requires = \
['dash>=2.0.0,<3.0.0',
 'flake8>=4.0.1,<5.0.0',
 'krakenex>=2.1.0,<3.0.0',
 'pandas>=1.3.4,<2.0.0',
 'pykrakenapi>=0.2.3,<0.3.0']

setup_kwargs = {
    'name': 'practica-kraken',
    'version': '0.1.1',
    'description': '',
    'long_description': None,
    'author': 'Your Name',
    'author_email': 'you@example.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
